# Kitchen Microwave

[![Build Status](https://img.shields.io/travis/socrata-platform/kitchen-microwave.svg)][travis]
[![Gem Version](https://img.shields.io/gem/v/kitchen-microwave.svg)][rubygems]

[travis]: https://travis-ci.org/socrata-platform/kitchen-microwave
[rubygems]: https://rubygems.org/gems/kitchen-microwave

Opinionated, config-less Test Kitchen!

This gem wraps Test Kitchen and dynamically configures it based on the content of the cookbook under test.

## Installation

Add this line to your application's Gemfile:

```ruby
gem 'kitchen-microwave'
```

And then execute:

    $ (chef exec) bundle install

Or install it yourself as:

    $ (chef exec) gem install kitchen-microwave

## Usage

The Microwave CLI accepts all the same commands and options as Test Kitchen, e.g. `microwave list`, `microwave converge`, etc.

For debugging purposes, running any Microwave command results in its YAML config being written out to `.kitchen/.microwave.yml`.

By default, Microwave will operate without any `.kitchen.yml` file present at all. It will generate the following config sections, each of which can be overridden in a `.kitchen.yml`:

***microwave***

The only config section unique to Microwave, this is used to set specific platforms (names + versions + Chef releases) to exclude from Test Kitchen builds, e.g.:

```
microwave:
  excludes:
    - name: ubuntu
      version: 14.04
      chef: 14
    - name: centos
      version: 7
    - name: fedora
```

Any field that is not specified will be interpreted as a wildcard.

***driver***

By default, Microwave uses the [Dokken](https://github.com/someara/kitchen-dokken) driver running in privileged mode.

***transport***

By default, Microwave uses the [Dokken](https://github.com/someara/kitchen-dokken) transport.

***provisioner***

By default, Microwave uses the [Dokken](https://github.com/someara/kitchen-dokken) provisioner.

***verifier***

By default, Microwave uses the [Inspec](https://www.inspec.io) verifier.

***platforms***

By default, Microwave generates a set of platforms based on the `supports` and `chef_version` fields in a cookbooks `metadata.rb`. For example:

```
chef_version '~> 13'
chef_version '~> 14'
supports 'ubuntu', '< 18.04'
supports 'centos', '~> 7'
```

results in the following set of platforms:

```
ubuntu-16.04-chef-14
ubuntu-14.04-chef-14
centos-7-chef-14
ubuntu-16.04-chef-13
ubuntu-14.04-chef-13
centos-7-chef-13
```

Each platform is given a set of `intermediate_instructions` and `pid_one_command` for the appropriate init system, e.g. installing and starting Systemd in Ubuntu 18.04.

Only certain platforms, versions, and Chefs are supported:

- Amazon Linux (2, 1)
- CentOS (7, 6)
- Debian (10, 9)
- Fedora (30)
- Ubuntu (18.04, 16.04, 14.04)

- Chef (14, 13, 12)

Any others specified in the metadata are ignored.

***suites***

The set of suites is generated from any recipes found in the test wrapper cookbook that do not start with an underscore. For example, these test recipes:

```shell
$ ls test/fixtures/cookbooks/test/recipes 
_prep.rb   default.rb nginx.rb
```

will result in this suites config:

```
suites:
  - name: default
    run_list:
      - recipe[test::default]
  - name: nginx
    run_list:
      - recipe[test::nginx]
```

## Contributing

Contributions can be submitted via GitHub pull requests. See [this article](https://help.github.com/articles/about-pull-requests/) if you're not familiar with GitHub Pull Requests. In brief:

1. Fork the project's repo in GitHub.
2. Create your feature branch (`git checkout -b my-new-feature`).
3. Add code and tests for the new feature.
4. Ensure all tests pass (`bundle exec rake` + `cd test; chef exec microwave test`).
5. Bump the version in `lib/kitchen/microwave/version.rb` in accordance with [semver](http://semver.org).
6. Add a brief description of the change to `CHANGELOG.md`.
7. Commit your changes (`git commit -am 'Add some feature'`).
8. Push the branch to GitHub (`git push origin my-new-feature`).
9. Create a new pull request.
10. Ensure the build process for the pull request succeeds.
11. Enjoy life until the change can be reviewed by a maintainer.
